@@ -59,8 +59,10 @@ rescale_none <- function(x, ...) {
 censor <- function(x, range = c(0, 1), only.finite = TRUE) {
   force(range)
   finite <- if (only.finite) is.finite(x) else TRUE
-  x[finite & x < range[1]] <- NA
-  x[finite & x > range[2]] <- NA
+  # Assign NA - this makes sure that, even if all elements are
+  # replaced with NA, it stays numeric (and isn't changed to logical)
+  x[finite & x < range[1]] <- NA_real_
+  x[finite & x > range[2]] <- NA_real_
   x
 }
 
@@ -122,10 +124,63 @@ expand_range <- function(range, mul = 0, add = 0, zero_width = 1) {
   }
 }
 
-#' Determine if range of vector is FP 0.
-#' 
+#' Determine if range of vector is close to zero, with a specified tolerance
+#'
+#' The machine epsilon is the difference between 1.0 and the next number
+#' that can be represented by the machine. By default, this function
+#' uses epsilon * 100 as the tolerance. First it scales the values so that
+#' they have a mean of 1, and then it checks if the difference between
+#' them is larger than the tolerance.
+#'
+#' @examples
+#' eps <- .Machine$double.eps
+#' zero_range(c(1, 1 + eps))       # TRUE
+#' zero_range(c(1, 1 + 99 * eps))  # TRUE
+#' zero_range(c(1, 1 + 101 * eps)) # FALSE - Crossed the tol threshold
+#' zero_range(c(1, 1 + 2 * eps), tol = eps) # FALSE - Changed tol
+#'
+#' # Scaling up or down all the values has no effect since the values
+#' # are rescaled to 1 before checking against tol
+#' zero_range(100000 * c(1, 1 + eps))        # TRUE
+#' zero_range(100000 * c(1, 1 + 200 * eps))  # FALSE
+#' zero_range(.00001 * c(1, 1 + eps))        # TRUE
+#' zero_range(.00001 * c(1, 1 + 200 * eps))  # FALSE
+#'
+#' # NA values
+#' zero_range(c(1, NA))   # NA
+#' zero_range(c(1, NaN))  # NA
+#'
+#' # Infinite values
+#' zero_range(c(1, Inf))     # FALSE
+#' zero_range(c(-Inf, Inf))  # FALSE
+#' zero_range(c(Inf, Inf))   # TRUE
+#'
 #' @export
 #' @param x numeric range: vector of length 2
-zero_range <- function(x) {
-  length(x) == 1 || isTRUE(all.equal(x[1] - x[2], 0))
+#' @param tol A value specifying the tolerance. Defaults to
+#'   \code{.Machine$double.eps * 100}.
+#' @return logical \code{TRUE} if the relative difference of the endpoints of
+#' the range are not distinguishable from 0.
+zero_range <- function(x, tol = .Machine$double.eps * 100) {
+  if (length(x) == 1) return(TRUE)
+  if (length(x) != 2) stop("x must be length 1 or 2")
+  if (any(is.na(x)))  return(NA)
+
+  # Special case: if they are equal as determined by ==, then there
+  # is zero range. Also handles (Inf, Inf) and (-Inf, -Inf)
+  if (x[1] == x[2]) return(TRUE)
+
+  # If we reach this, then x must be (-Inf, Inf) or (Inf, -Inf)
+  if (all(is.infinite(x))) return(FALSE)
+
+  # Take the smaller (in magnitude) value of x, and use it as the scaling
+  # factor.
+  m <- min(abs(x))
+
+  # If we get here, then exactly one of the x's is 0. Return FALSE
+  if (m == 0) return(FALSE)
+
+  # If x[1] - x[2] (scaled to 1) is smaller than tol, then return
+  # TRUE; otherwise return FALSE
+  abs((x[1] - x[2])/m) < tol
 }
